@@ -6,7 +6,6 @@ A wheel is a built archive format.
 import csv
 import hashlib
 import os
-import sys
 import subprocess
 
 try:
@@ -28,19 +27,10 @@ from distutils.sysconfig import get_python_version
 from distutils import log as logger
 import shutil
 
-from .util import get_abbr_impl, get_impl_ver, native
+from .util import get_abbr_impl, get_impl_ver, native, open_for_csv
 from .archive import archive_wheelfile
 from .pkginfo import read_pkg_info, write_pkg_info
 
-def open_for_csv(name, mode):
-    if sys.version_info[0] < 3:
-        nl = {}
-        bin = 'b'
-    else:
-        nl = { 'newline': '' }
-        bin = ''
-    return open(name, mode + bin, **nl)
-
 def safer_name(name):
     return safe_name(name).replace('-', '_')
 
@@ -53,7 +43,7 @@ class bdist_wheel(Command):
 
     description = 'create a wheel distribution'
 
-    user_options = [('bdist-dir=', 'd',
+    user_options = [('bdist-dir=', 'b',
                      "temporary directory for creating the distribution"),
                     ('plat-name=', 'p',
                      "platform name to embed in generated filenames "
@@ -196,6 +186,9 @@ class bdist_wheel(Command):
                                          '%s.dist-info' % self.wheel_dist_name)
         self.egg2dist(self.egginfo_dir,
                       self.distinfo_dir)
+                
+        metadata_path = os.path.join(self.distinfo_dir, 'METADATA')
+        self.add_requirements(metadata_path)
 
         self.write_wheelfile(self.distinfo_dir)
 
@@ -220,11 +213,11 @@ class bdist_wheel(Command):
             else:
                 rmtree(self.bdist_dir)
 
-    def write_wheelfile(self, wheelfile_base, packager='bdist_wheel'):
+    def write_wheelfile(self, wheelfile_base, generator='bdist_wheel'):
         from email.message import Message
         msg = Message()
         msg['Wheel-Version'] = '0.1'  # of the spec
-        msg['Packager'] = packager
+        msg['Generator'] = generator
         msg['Root-Is-Purelib'] = str(self.root_is_purelib).lower()
         wheelfile_path = os.path.join(wheelfile_base, 'WHEEL')
         logger.info('creating %s', wheelfile_path)
@@ -263,11 +256,8 @@ class bdist_wheel(Command):
         return " (%s)" % ','.join(requires_dist)
 
     def _pkginfo_to_metadata(self, egg_info_path, pkginfo_path):
-        # XXX does Requires: become Requires-Dist: ?
-        # (very few source packages include Requires: (644) or
-        # Requires-Dist: (5) in PKG-INFO); packaging treats both identically
         pkg_info = read_pkg_info(pkginfo_path)
-        pkg_info.replace_header('Metadata-Version', '1.2')
+        pkg_info.replace_header('Metadata-Version', '1.3')
         requires_path = os.path.join(egg_info_path, 'requires.txt')
         if os.path.exists(requires_path):
             requires = open(requires_path).read()
@@ -282,7 +272,44 @@ class bdist_wheel(Command):
                     pkg_info['Requires-Dist'] = parsed_requirement.key + \
                         spec + condition
         return pkg_info
-
+    
+    def setupcfg_requirements(self):
+        """Generate requirements from setup.cfg as 
+        ('Requires-Dist', 'requirement; qualifier') tuples. From a metadata
+        section in setup.cfg:
+        
+        [metadata]
+        requires_dist = requirement; qualifier
+            another; qualifier2
+            unqualified
+            
+        Yields
+        
+        ('Requires-Dist', 'requirement; qualifier'),
+        ('Requires-Dist', 'another; qualifier2'),
+        ('Requires-Dist', 'unqualified')
+        """
+        requires = []
+        try:
+            metadata = self.distribution.get_option_dict('metadata')
+            requires = metadata['requires_dist']
+            for req in requires[1].splitlines():
+                req = req.strip()
+                if not req:
+                    continue
+                yield ('Requires-Dist', req)                 
+        except KeyError:
+            return
+    
+    def add_requirements(self, metadata_path):
+        """Add additional requirements from setup.cfg to file metadata_path"""
+        additional = list(self.setupcfg_requirements())
+        if not additional: return
+        pkg_info = read_pkg_info(metadata_path)
+        for k, v in additional:
+            pkg_info[k] = v
+        write_pkg_info(metadata_path, pkg_info)
+        
     def egg2dist(self, egginfo_path, distinfo_path):
         """Convert an .egg-info directory into a .dist-info directory"""
         def adios(p):
@@ -330,7 +357,7 @@ class bdist_wheel(Command):
             # delete dependency_links if it is only whitespace
             dependency_links = os.path.join(distinfo_path, 'dependency_links.txt')
             if not open(dependency_links, 'r').read().strip(): 
-                adios(dependency_links)
+                adios(dependency_links)        
 
         write_pkg_info(os.path.join(distinfo_path, 'METADATA'), pkg_info)
 
